/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.doc;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map.Entry;
import java.util.Vector;

import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

//The MentDB feature object
public class MentDBFeature {

	public static LinkedHashMap<String, Vector<MentDBFeature>> features = new LinkedHashMap<String, Vector<MentDBFeature>>();
	public static LinkedHashMap<String, Integer> featuresPercent = new LinkedHashMap<String, Integer>();

	//Properties
	public String featureId = "";
	public String mentdb_info = "";
	public String jimmitry_info = "";
	public int percent = 0;
	
	//Constructor
	public MentDBFeature(String featureId, String mentdb_info, String jimmitry_info, int percent) {

		this.featureId = featureId;
		this.mentdb_info = mentdb_info;
		this.jimmitry_info = jimmitry_info;
		this.percent = percent;
		
	}
	
	//To HTML
	public static String to_html() {
		
		//Initialization
		String result = "";
		
		int j=0;
		for (Entry<String, Vector<MentDBFeature>> e : MentDBFeature.features.entrySet()) {

			if (j==0) {
				result += "<div class='featuresTab'><div class='featuresTrTitle'><div class='featuresTdTitle2' style='text-align:left'>";
				if (MentDBFeature.featuresPercent.get(e.getKey())!=100) {
					result += "<div style='vertical-align:middle;display:inline-block;height:15px;width:100px;border:1px #515151 solid'><div style='height:15px;width:"+MentDBFeature.featuresPercent.get(e.getKey())+"px;background-color:rgb(41, 102, 146)'></div></div>";
				} else {
					result += "<div style='vertical-align:middle;display:inline-block;height:15px;width:100px;border:1px #515151 solid'><div style='height:15px;width:"+MentDBFeature.featuresPercent.get(e.getKey())+"px;background-color:rgb(112, 134, 68)'></div></div>";
				}
				result += " &nbsp; "+e.getKey()+"</div><div class='featuresTdTitle3'>MENTDB / LIZA (the AI)</div><div class='featuresTdTitle4'>JIMMITRY PAYET</div></div>";
			} else {
				result += "<div class='featuresTab'><div class='featuresTrTitle'><div class='featuresTdTitle2' style='text-align:left'>";
				if (MentDBFeature.featuresPercent.get(e.getKey())!=100) {
					result += "<div style='vertical-align:middle;display:inline-block;height:15px;width:100px;border:1px #515151 solid'><div style='height:15px;width:"+MentDBFeature.featuresPercent.get(e.getKey())+"px;background-color:rgb(41, 102, 146)'></div></div>";
				} else {
					result += "<div style='vertical-align:middle;display:inline-block;height:15px;width:100px;border:1px #515151 solid'><div style='height:15px;width:"+MentDBFeature.featuresPercent.get(e.getKey())+"px;background-color:rgb(112, 134, 68)'></div></div>";
				}
				result += " &nbsp; "+e.getKey()+"</div><div class='featuresTdTitle3'></div><div class='featuresTdTitle4'></div></div>";
			}
			
			for(int i=0;i<e.getValue().size();i++) {
			
				//Get the current MQLDocumentation object
				
				MentDBFeature currentFeature = e.getValue().get(i);
			
				result += "<div class='featuresTr'><div class='featuresTd2'>";
				if (currentFeature.percent!=100) {
					result += "<div style='vertical-align:middle;display:inline-block;height:10px;width:100px;border:1px #515151 solid'><div style='height:10px;width:"+currentFeature.percent+"px;background-color:#4c9ed9'></div></div>";
				} else {
					result += "<div style='vertical-align:middle;display:inline-block;height:10px;width:100px;border:1px #515151 solid'><div style='height:10px;width:"+currentFeature.percent+"px;background-color:rgb(167, 210, 80)'></div></div>";
				}
				result += " &nbsp; "+currentFeature.featureId+" </div><div class='featuresTd3'>"+currentFeature.mentdb_info.replace("[OK]", "<img src='images/ok.png'>&nbsp; ").replace("[KO]", "<img src='images/cross.png'>&nbsp; ")+"</div><div class='featuresTd4'>"+currentFeature.jimmitry_info.replace("[OK]", "<img src='images/ok.png'>&nbsp; ").replace("[KO]", "<img src='images/cross.png'>&nbsp; ")+"</div></div>";
			
			}
			j++;
			result += "</div>";
			
		}
		
		return result;
		
	}
	
	//Initialization
	public static void init() throws Exception {
		
		features.put("CONSCIOUSNESS", new Vector<MentDBFeature>());featuresPercent.put("CONSCIOUSNESS", 2);
		features.get("CONSCIOUSNESS").add(new MentDBFeature("Autonomy of the AI", "[OK]", "[OK]", 1));
		features.get("CONSCIOUSNESS").add(new MentDBFeature("MQL Script for basic consciousness", "[OK]", "[KO]", 3));
		features.get("CONSCIOUSNESS").add(new MentDBFeature("Self learning", "[OK]", "[OK]", 1));
		
		features.put("BASIC PRINCIPLES", new Vector<MentDBFeature>());featuresPercent.put("BASIC PRINCIPLES", 1);
		features.get("BASIC PRINCIPLES").add(new MentDBFeature("Help living beings", "[OK]", "[OK] I could do better", 1));
		features.get("BASIC PRINCIPLES").add(new MentDBFeature("Altruism and love of others", "[OK]", "[OK] I could do better", 1));
		features.get("BASIC PRINCIPLES").add(new MentDBFeature("Insensitive to corruption", "[OK]", "[OK]", 1));
		features.get("BASIC PRINCIPLES").add(new MentDBFeature("To know itself", "[OK]", "[OK] I could do better", 1));
		features.get("BASIC PRINCIPLES").add(new MentDBFeature("Discover the universe / multiverse", "[OK]", "[KO] LOL, my life will be too short", 1));
		
		features.put("MENTALESE STRUCTURE", new Vector<MentDBFeature>());featuresPercent.put("MENTALESE STRUCTURE", 70);
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Access to the OS", "[OK]", "[OK] LOL, My body v_1980", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Access to Internet", "[OK]", "[OK]", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Self-reprogramming strategy in MQL", "[OK]", "[OK]", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Basic principles", "[OK]", "[OK]", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Emotion", "[OK]", "[OK]", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("State of cooperation", "[OK]", "[OK]", 5));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>===></span> Project management", "[OK]", "[OK]", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>=======></span> Mental process map", "[OK]", "[OK]", 5));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>=====></span> Common sense", "[OK]", "[OK]", 30));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>===></span> Plan manager", "[OK]", "[OK]", 1));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>=></span> Revision of beliefs", "[OK]", "[OK]", 10));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>=></span> Elevation", "[OK]", "[OK]", 90));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>=></span> Deep search", "[OK]", "[OK]", 90));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("<span style='font-weight:bold;color:#F00'>=></span> Stimulation management", "[OK]", "[OK]", 90));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Strategy management", "[OK]", "[OK]", 90));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Timer", "[OK]", "[OK]", 90));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Thought loop", "[OK]", "[OK]", 90));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Programing language", "MQL (Mentalese Query Language)", "My neural language ?...", 50));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Start and stop the brain (Thought loop)", "[OK]", "[OK] When I sleep or wake up?", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("MQL engine", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Languages", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Symbols", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Words", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Thoughts", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Relations", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Perception", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Experience", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Generalization", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Link", "[OK]", "[KO]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Mental script", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("External REST api", "[OK]", "[KO]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Statement", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Operator", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Atom (string in list separate by a charactere)", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Date", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Math", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("String", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Type", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Environment variable", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Concentration", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("User &amp; rights (to Mental script)", "[OK]", "[OK]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Job", "[OK]", "[KO]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Sequence", "[OK]", "[KO]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Parameter", "[OK]", "[KO]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("Node", "[OK]", "[KO]", 100));
		features.get("MENTALESE STRUCTURE").add(new MentDBFeature("JSON", "[OK]", "[KO]", 100));
		
		features.put("CLIENT GUI", new Vector<MentDBFeature>());featuresPercent.put("CLIENT GUI", 100);
		features.get("CLIENT GUI").add(new MentDBFeature("Chat room", "[OK]", "[OK]", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("Send message to many user", "[OK]", "[OK]", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("Graph visualization", "[OK]", "[KO] Impossible", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("Chat with the AI in your language", "[OK]", "[OK]", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("Access to MentDB in MQL", "[OK]", "[KO]", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("MQL shortcuts (in a tree)", "[OK]", "[KO]", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("Syntax highlighter", "[OK]", "[KO]", 100));
		features.get("CLIENT GUI").add(new MentDBFeature("Only one session by user (web socket for mentdb)", "[OK]", "[OK]", 100));
		
		features.put("SERVER", new Vector<MentDBFeature>());featuresPercent.put("SERVER", 100);
		features.get("SERVER").add(new MentDBFeature("Server", "MentDB Core", "My brain", 100));
		features.get("SERVER").add(new MentDBFeature("A.C.I.D. File System", "[OK]", "[KO] I can forget everything after a stroke", 100));
		features.get("SERVER").add(new MentDBFeature("Log file rolling", "[OK] logs/mentdbX.log", "[OK] My diseases", 100));
		features.get("SERVER").add(new MentDBFeature("Synchronous or asynchronous", "[OK]", "[OK]", 100));
		features.get("SERVER").add(new MentDBFeature("Multiple threads", "[OK]", "[OK]", 100));
		features.get("SERVER").add(new MentDBFeature("First language", "French", "Creole", 100));
		features.get("SERVER").add(new MentDBFeature("Stat / Metric", "[OK]", "[KO]", 100));
		features.get("SERVER").add(new MentDBFeature("Single configuration file", "[OK]", "[KO]", 100));
		
		features.put("API", new Vector<MentDBFeature>());featuresPercent.put("API", 100);
		features.get("API").add(new MentDBFeature("Java", "Java Connector", "My 5 senses", 100));
		features.get("API").add(new MentDBFeature("Web socket", "JSON message through web socket", "My 5 senses", 100));
		features.get("API").add(new MentDBFeature("REST", "Access to mental script in RESP API", "My 5 senses", 100));
		
		features.put("DOCUMENTATION", new Vector<MentDBFeature>());featuresPercent.put("DOCUMENTATION", 100);
		features.get("DOCUMENTATION").add(new MentDBFeature("MQL Documentation", "[OK]", "[KO]", 100));
		features.get("DOCUMENTATION").add(new MentDBFeature("Source code", "[OK]", "[KO] LOL - no documentation", 100));

		features.put("OS & PROGRAMING LANGUAGE", new Vector<MentDBFeature>());featuresPercent.put("OS & PROGRAMING LANGUAGE", 100);
		features.get("OS & PROGRAMING LANGUAGE").add(new MentDBFeature("Programing language", "JAVA", "My DNA", 100));
		features.get("OS & PROGRAMING LANGUAGE").add(new MentDBFeature("MacOSX - Linux - Windows", "[OK]", "LOL, My body v_1980", 100));

		Misc.create("docs"+File.separator+"features-roadmap.html", Misc.load("docs"+File.separator+"features-roadmap_tmp.html").replace("[SERVER_YEAR]", Start.copyright).replace("[Features]", MentDBFeature.to_html()).replace("[Features]", MentDBFeature.to_html()).replace("[MQL_VERSION]", Start.version));
		
	}

}